@@ -133,12 +133,12 @@ struct _GossipAppPriv {
 	GtkTooltips           *tooltips;
 
 	/* Menu widgets */
-	GtkWidget             *actions_connect;
-	GtkWidget             *actions_disconnect;
-	GtkWidget             *actions_group_chat;
-	GtkWidget             *actions_group_chat_menu;
-	GtkWidget             *actions_group_chat_sep;
-	GtkWidget             *actions_group_chat_join;
+	GtkWidget             *chat_connect;
+	GtkWidget             *chat_disconnect;
+	GtkWidget             *room;
+	GtkWidget             *room_menu;
+	GtkWidget             *room_sep;
+	GtkWidget             *room_join_favorites;
 	GtkWidget             *edit_context;
 	GtkWidget             *edit_context_separator;
 
@@ -192,10 +192,6 @@ static gboolean        app_main_window_delete_event_cb        (GtkWidget
 static gboolean        app_main_window_key_press_event_cb     (GtkWidget                *window,
 							       GdkEventKey              *event,
 							       GossipApp                *app);
-static void            app_favorite_chatroom_menu_join_cb     (GossipChatroomProvider   *provider,
-							       GossipChatroomJoinResult  result,
-							       GossipChatroomId          id,
-							       gpointer                  user_data);
 static void            app_favorite_chatroom_menu_activate_cb (GtkMenuItem              *menu_item,
 							       GossipChatroom           *chatroom);
 static void            app_favorite_chatroom_menu_update      (void);
@@ -210,25 +206,25 @@ static void            app_favorite_chatroom_menu_removed_cb  (GossipChatroomMan
 							       GossipChatroom           *chatroom,
 							       gpointer                  user_data);
 static void            app_favorite_chatroom_menu_setup       (void);
-static void            app_actions_quit_cb                    (GtkWidget                *window,
+static void            app_chat_quit_cb                       (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_connect_cb                 (GtkWidget                *window,
+static void            app_chat_connect_cb                    (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_disconnect_cb              (GtkWidget                *window,
+static void            app_chat_disconnect_cb                 (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_new_message_cb             (GtkWidget                *widget,
+static void            app_chat_new_message_cb                (GtkWidget                *widget,
 							       GossipApp                *app);
-static void            app_actions_history_cb                 (GtkWidget                *window,
+static void            app_chat_history_cb                    (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_group_chat_new_cb          (GtkWidget                *window,
+static void            app_room_join_new_cb                   (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_group_chat_join_cb         (GtkWidget                *window,
+static void            app_room_join_favorites_cb             (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_group_chat_rooms_cb        (GtkWidget                *window,
+static void            app_room_manage_favorites_cb           (GtkWidget                *window,
 							       GossipApp                *app);
-static void            app_actions_add_contact_cb             (GtkWidget                *widget,
+static void            app_chat_add_contact_cb                (GtkWidget                *widget,
 							       GossipApp                *app);
-static void            app_actions_show_offline_cb            (GtkCheckMenuItem         *item,
+static void            app_chat_show_offline_cb               (GtkCheckMenuItem         *item,
 							       GossipApp                *app);
 static gboolean        app_edit_button_press_event_cb         (GtkWidget                *widget,
 							       GdkEventButton           *event,
@@ -283,6 +279,7 @@ static void            app_status_icon_popup_menu_cb          (GtkStatusIcon
 							       GossipApp                *app);
 static void            app_status_icon_create_menu            (void);
 static void            app_status_icon_create                 (void);
+static gboolean        app_status_icon_check_embedded_cb      (gpointer                  user_data);
 static void            app_status_icon_update_tooltip         (void);
 static void            app_status_icon_flash_start            (void);
 static void            app_status_icon_flash_maybe_stop       (void);
@@ -310,9 +307,9 @@ static GossipPresence *app_get_effective_presence             (void);
 static void            app_set_away                           (const gchar              *status);
 static GdkPixbuf *     app_get_current_status_pixbuf          (void);
 static GossipPresenceState
-		       app_get_current_state                  (void);
+                       app_get_current_state                  (void);
 static GossipPresenceState
-		       app_get_previous_state                 (void);
+                       app_get_previous_state                 (void);
 static void            app_presence_updated                   (void);
 static void            app_status_clear_away                  (void);
 static void            app_presence_chooser_changed_cb        (GtkWidget                *chooser,
@@ -325,10 +322,9 @@ static gboolean        app_window_configure_event_cb          (GtkWidget
 							       GossipApp                *app);
 static void            app_status_flash_start                 (void);
 static void            app_status_flash_stop                  (void);
-static void            app_chatroom_auto_connect_update_cb    (GossipChatroomManager    *manager,
+static void            app_chatroom_auto_connect_cb           (GossipChatroomManager    *manager,
 							       GossipChatroomProvider   *provider,
 							       GossipChatroom           *chatroom,
-							       GossipChatroomJoinResult  result,
 							       gpointer                  user_data);
 static void            app_event_added_cb                     (GossipEventManager       *manager,
 							       GossipEvent              *event,
@@ -444,7 +440,7 @@ app_finalize (GObject *object)
 					      NULL);
 
 	g_signal_handlers_disconnect_by_func (priv->chatroom_manager,
-					      app_chatroom_auto_connect_update_cb,
+					      app_chatroom_auto_connect_cb,
 					      NULL);
 
 	g_signal_handlers_disconnect_by_func (priv->event_manager,
@@ -481,14 +477,12 @@ app_setup (GossipSession        *session,
 	GossipConf     *conf;
 	GladeXML       *glade;
 	GtkWidget      *sw;
-	GtkToolItem    *item;
 	GtkWidget      *show_offline_widget;
-	GtkRequisition  req;
+	GtkToolItem    *item;
 	gchar          *str;
 	gboolean        show_offline;
 	gboolean        show_avatars;
 	gboolean        compact_contact_list;
-	gboolean        hidden;
 	gint            x, y, w, h;
 
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Beginning...");
@@ -533,6 +527,7 @@ app_setup (GossipSession        *session,
 
 	gossip_sound_init (priv->session);
 
+	/* Set up signals */
 	g_signal_connect (priv->session, "protocol-connecting",
 			  G_CALLBACK (app_session_protocol_connecting_cb),
 			  NULL);
@@ -553,8 +548,8 @@ app_setup (GossipSession        *session,
 			  G_CALLBACK (app_session_get_password_cb),
 			  NULL);
 
-	g_signal_connect (priv->chatroom_manager, "chatroom-auto-connect-update",
-			  G_CALLBACK (app_chatroom_auto_connect_update_cb),
+	g_signal_connect (priv->chatroom_manager, "chatroom-auto-connect",
+			  G_CALLBACK (app_chatroom_auto_connect_cb),
 			  NULL);
 
 	g_signal_connect (priv->event_manager, "event-added",
@@ -572,12 +567,12 @@ app_setup (GossipSession        *session,
 				       "main_window", &priv->window,
 				       "main_vbox", &priv->main_vbox,
 				       "errors_vbox", &priv->errors_vbox,
-				       "actions_connect", &priv->actions_connect,
-				       "actions_disconnect", &priv->actions_disconnect,
-				       "actions_show_offline", &show_offline_widget,
-				       "actions_group_chat", &priv->actions_group_chat,
-				       "actions_group_chat_sep", &priv->actions_group_chat_sep,
-				       "actions_group_chat_join", &priv->actions_group_chat_join,
+				       "chat_connect", &priv->chat_connect,
+				       "chat_disconnect", &priv->chat_disconnect,
+				       "chat_show_offline", &show_offline_widget,
+				       "room", &priv->room,
+				       "room_sep", &priv->room_sep,
+				       "room_join_favorites", &priv->room_join_favorites,
 				       "edit_context", &priv->edit_context,
 				       "edit_context_separator", &priv->edit_context_separator,
 				       "presence_toolbar", &priv->presence_toolbar,
@@ -590,16 +585,16 @@ app_setup (GossipSession        *session,
 			      "main_window", "delete_event", app_main_window_delete_event_cb,
 			      "main_window", "configure_event", app_window_configure_event_cb,
 			      "main_window", "key_press_event", app_main_window_key_press_event_cb,
-			      "actions_quit", "activate", app_actions_quit_cb,
-			      "actions_connect", "activate", app_actions_connect_cb,
-			      "actions_disconnect", "activate", app_actions_disconnect_cb,
-			      "actions_new_message", "activate", app_actions_new_message_cb,
-			      "actions_history", "activate", app_actions_history_cb,
-			      "actions_group_chat_new", "activate", app_actions_group_chat_new_cb,
-			      "actions_group_chat_join", "activate", app_actions_group_chat_join_cb,
-			      "actions_group_chat_rooms", "activate", app_actions_group_chat_rooms_cb,
-			      "actions_add_contact", "activate", app_actions_add_contact_cb,
-			      "actions_show_offline", "toggled", app_actions_show_offline_cb,
+			      "chat_quit", "activate", app_chat_quit_cb,
+			      "chat_connect", "activate", app_chat_connect_cb,
+			      "chat_disconnect", "activate", app_chat_disconnect_cb,
+			      "chat_new_message", "activate", app_chat_new_message_cb,
+			      "chat_history", "activate", app_chat_history_cb,
+			      "room_join_new", "activate", app_room_join_new_cb,
+			      "room_join_favorites", "activate", app_room_join_favorites_cb,
+			      "room_manage_favorites", "activate", app_room_manage_favorites_cb,
+			      "chat_add_contact", "activate", app_chat_add_contact_cb,
+			      "chat_show_offline", "toggled", app_chat_show_offline_cb,
 			      "edit", "button-press-event", app_edit_button_press_event_cb,
 			      "edit_accounts", "activate", app_edit_accounts_cb,
 			      "edit_personal_information", "activate", app_edit_personal_information_cb,
@@ -681,11 +676,6 @@ app_setup (GossipSession        *session,
 	gtk_container_add (GTK_CONTAINER (sw),
 			   GTK_WIDGET (priv->contact_list));
 
-	/* Set up notification area / tray. */
-	gossip_debug (DEBUG_DOMAIN_SETUP, "Configuring notification area widgets");
-	app_status_icon_create_menu ();
-	app_status_icon_create ();
-
 	/* Load user-defined accelerators. */
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Configuring accels");
 	app_accels_load ();
@@ -713,6 +703,11 @@ app_setup (GossipSession        *session,
 		gtk_window_move (GTK_WINDOW (priv->window), x, y);
 	}
 
+	/* Set up notification area / tray. */
+	gossip_debug (DEBUG_DOMAIN_SETUP, "Configuring notification area widgets");
+	app_status_icon_create_menu ();
+	app_status_icon_create ();
+
 	/* Set up current presence. */
 	app_presence_updated ();
 
@@ -751,39 +746,9 @@ app_setup (GossipSession        *session,
 		      NULL);
 
 	/* Set window to be hidden. If doesn't have status icon, show window
-	 * and mask "actions_hide_list".
+	 * and mask "chat_hide_list".
 	 */
-	if (!gtk_status_icon_is_embedded (priv->status_icon)) {
-		hidden = FALSE;
-	} else {
-		gossip_conf_get_bool (conf,
-				      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN,
-				      &hidden);
-	}
-
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (show_offline_widget),
-					show_offline);
-
-	if (!hidden) {
-		gtk_widget_show (priv->window);
-	} else {
-		gdk_notify_startup_complete ();
-	}
-
-	/* This is a hack that sets the minimal size of the window so it doesn't
-	 * allow resizing to a smaller width than the menubar takes up. We must
-	 * set a minimal size, otherwise the window won't shrink beyond the
-	 * longest string in the roster, which will cause the ellipsizing cell
-	 * renderer not to work. FIXME: needs to update this on theme/font
-	 * change.
-	 */
-	gtk_widget_size_request (priv->window, &req);
-	req.width = MAX (req.width, MIN_WIDTH);
-	gtk_widget_set_size_request (priv->window, req.width, -1);
-
-	app_connection_items_update ();
-
-	gossip_debug (DEBUG_DOMAIN_SETUP, "Complete!");
+	g_timeout_add (200, app_status_icon_check_embedded_cb, NULL);
 }
 
 static gboolean
@@ -999,18 +964,6 @@ app_main_window_key_press_event_cb (GtkWidget   *window,
 	return FALSE;
 }
 
-static void
-app_favorite_chatroom_menu_join_cb (GossipChatroomProvider   *provider,
-				    GossipChatroomJoinResult  result,
-				    GossipChatroomId          id,
-				    gpointer                  user_data)
-{
-	if (result == GOSSIP_CHATROOM_JOIN_OK ||
-	    result == GOSSIP_CHATROOM_JOIN_ALREADY_OPEN) {
-		gossip_group_chat_new (provider, id);
-	}
-}
-
 static void
 app_favorite_chatroom_menu_activate_cb (GtkMenuItem    *menu_item,
 					GossipChatroom *chatroom)
@@ -1023,10 +976,7 @@ app_favorite_chatroom_menu_activate_cb (GtkMenuItem    *menu_item,
 	account = gossip_chatroom_get_account (chatroom);
 	provider = gossip_session_get_chatroom_provider (session, account);
 
-	gossip_chatroom_provider_join (provider,
-				       chatroom,
-				       app_favorite_chatroom_menu_join_cb,
-				       NULL);
+	gossip_group_chat_new (provider, chatroom);
 }
 
 static void
@@ -1064,12 +1014,12 @@ app_favorite_chatroom_menu_update (void)
 	g_list_free (chatrooms);
 
 	if (found) {
-		gtk_widget_show (priv->actions_group_chat_sep);
+		gtk_widget_show (priv->room_sep);
 	} else {
-		gtk_widget_hide (priv->actions_group_chat_sep);
+		gtk_widget_hide (priv->room_sep);
 	}
 
-	gtk_widget_set_sensitive (priv->actions_group_chat_join, found);
+	gtk_widget_set_sensitive (priv->room_join_favorites, found);
 }
 
 static gboolean
@@ -1101,7 +1051,7 @@ app_favorite_chatroom_menu_add (GossipChatroom *chatroom)
 			  G_CALLBACK (app_favorite_chatroom_menu_activate_cb),
 			  chatroom);
 
-	gtk_menu_shell_insert (GTK_MENU_SHELL (priv->actions_group_chat_menu),
+	gtk_menu_shell_insert (GTK_MENU_SHELL (priv->room_menu),
 			       menu_item, 3);
 
 	if (visible) {
@@ -1129,8 +1079,8 @@ app_favorite_chatroom_menu_added_cb (GossipChatroomManager *manager,
 	priv = GET_PRIV (app);
 
 	if (app_favorite_chatroom_menu_add (chatroom)) {
-		gtk_widget_show (priv->actions_group_chat_sep);
-		gtk_widget_set_sensitive (priv->actions_group_chat_join, TRUE);
+		gtk_widget_show (priv->room_sep);
+		gtk_widget_set_sensitive (priv->room_join_favorites, TRUE);
 	}
 }
 
@@ -1170,8 +1120,8 @@ app_favorite_chatroom_menu_setup (void)
 	priv = GET_PRIV (app);
 
 	chatrooms = gossip_chatroom_manager_get_chatrooms (priv->chatroom_manager, NULL);
-	priv->actions_group_chat_menu =
-		gtk_menu_item_get_submenu (GTK_MENU_ITEM (priv->actions_group_chat));
+	priv->room_menu =
+		gtk_menu_item_get_submenu (GTK_MENU_ITEM (priv->room));
 
 	for (l = chatrooms; l; l = l->next) {
 		if (app_favorite_chatroom_menu_add (l->data)) {
@@ -1182,10 +1132,10 @@ app_favorite_chatroom_menu_setup (void)
 	g_list_free (chatrooms);
 
 	if (!found) {
-		gtk_widget_hide (priv->actions_group_chat_sep);
+		gtk_widget_hide (priv->room_sep);
 	}
 
-	gtk_widget_set_sensitive (priv->actions_group_chat_join, found);
+	gtk_widget_set_sensitive (priv->room_join_favorites, found);
 
 	g_signal_connect (priv->chatroom_manager, "chatroom-favourite-update",
 			  G_CALLBACK (app_favorite_chatroom_menu_update_cb),
@@ -1199,8 +1149,8 @@ app_favorite_chatroom_menu_setup (void)
 }
 
 static void
-app_actions_quit_cb (GtkWidget *window,
-		     GossipApp *app)
+app_chat_quit_cb (GtkWidget *window,
+		  GossipApp *app)
 {
 	GossipAppPriv *priv;
 
@@ -1212,8 +1162,8 @@ app_actions_quit_cb (GtkWidget *window,
 }
 
 static void
-app_actions_connect_cb (GtkWidget *window,
-			GossipApp *app)
+app_chat_connect_cb (GtkWidget *window,
+		     GossipApp *app)
 {
 	g_return_if_fail (GOSSIP_IS_APP (app));
 
@@ -1221,8 +1171,8 @@ app_actions_connect_cb (GtkWidget *window,
 }
 
 static void
-app_actions_disconnect_cb (GtkWidget *window,
-			   GossipApp *app)
+app_chat_disconnect_cb (GtkWidget *window,
+			GossipApp *app)
 {
 	g_return_if_fail (GOSSIP_IS_APP (app));
 
@@ -1230,8 +1180,8 @@ app_actions_disconnect_cb (GtkWidget *window,
 }
 
 static void
-app_actions_new_message_cb (GtkWidget *widget,
-			    GossipApp *app)
+app_chat_new_message_cb (GtkWidget *widget,
+			 GossipApp *app)
 {
 	GossipAppPriv *priv;
 
@@ -1241,8 +1191,8 @@ app_actions_new_message_cb (GtkWidget *widget,
 }
 
 static void
-app_actions_history_cb (GtkWidget *widget,
-			GossipApp *app)
+app_chat_history_cb (GtkWidget *widget,
+		     GossipApp *app)
 {
 	GossipAppPriv *priv;
 
@@ -1252,8 +1202,8 @@ app_actions_history_cb (GtkWidget *widget,
 }
 
 static void
-app_actions_group_chat_new_cb (GtkWidget *window,
-			       GossipApp *app)
+app_room_join_new_cb (GtkWidget *window,
+		      GossipApp *app)
 {
 	GossipAppPriv *priv;
 
@@ -1263,8 +1213,8 @@ app_actions_group_chat_new_cb (GtkWidget *window,
 }
 
 static void
-app_actions_group_chat_join_cb (GtkWidget *window,
-				GossipApp *app)
+app_room_join_favorites_cb (GtkWidget *window,
+			    GossipApp *app)
 {
 	GossipAppPriv *priv;
 
@@ -1274,15 +1224,15 @@ app_actions_group_chat_join_cb (GtkWidget *window,
 }
 
 static void
-app_actions_group_chat_rooms_cb (GtkWidget *window,
-				 GossipApp *app)
+app_room_manage_favorites_cb (GtkWidget *window,
+			      GossipApp *app)
 {
 	gossip_chatrooms_window_show (NULL, FALSE);
 }
 
 static void
-app_actions_add_contact_cb (GtkWidget *widget,
-			    GossipApp *app)
+app_chat_add_contact_cb (GtkWidget *widget,
+			 GossipApp *app)
 {
 	GossipAppPriv *priv;
 
@@ -1292,8 +1242,8 @@ app_actions_add_contact_cb (GtkWidget *widget,
 }
 
 static void
-app_actions_show_offline_cb (GtkCheckMenuItem *item,
-			     GossipApp        *app)
+app_chat_show_offline_cb (GtkCheckMenuItem *item,
+			  GossipApp        *app)
 {
 	GossipAppPriv *priv;
 	gboolean       current;
@@ -1303,8 +1253,8 @@ app_actions_show_offline_cb (GtkCheckMenuItem *item,
 	current = gtk_check_menu_item_get_active (item);
 
 	gossip_conf_set_bool (gossip_conf_get (),
-			       GOSSIP_PREFS_CONTACTS_SHOW_OFFLINE,
-			       current);
+			      GOSSIP_PREFS_CONTACTS_SHOW_OFFLINE,
+			      current);
 
 	/* Turn off sound just while we alter the contact list. */
 	gossip_sound_set_enabled (FALSE);
@@ -1445,7 +1395,7 @@ app_session_protocol_connecting_cb (GossipSession  *session,
 
 	priv = GET_PRIV (app);
 
-	gossip_account_param_get (account, "id", &id, NULL);
+	id = gossip_account_get_id (account);
 	gossip_debug (DEBUG_DOMAIN_SESSION, "Connecting account:'%s'", id);
 
 	gossip_throbber_start (GOSSIP_THROBBER (priv->throbber));
@@ -1463,7 +1413,7 @@ app_session_protocol_connected_cb (GossipSession  *session,
 
 	priv = GET_PRIV (app);
 
-	gossip_account_param_get (account, "id", &id, NULL);
+	id = gossip_account_get_id (account);
 	gossip_debug (DEBUG_DOMAIN_SESSION, "Connected account:'%s'", id);
 
 	gossip_session_count_accounts (priv->session,
@@ -1527,7 +1477,7 @@ app_session_protocol_disconnected_cb (GossipSession  *session,
 
 	priv = GET_PRIV (app);
 
-	gossip_account_param_get (account, "id", &id, NULL);
+	id = gossip_account_get_id (account);
 	gossip_debug (DEBUG_DOMAIN_SESSION, "Disconnected account:'%s'", id);
 
 	gossip_session_count_accounts (priv->session,
@@ -1581,7 +1531,7 @@ app_session_protocol_error_cb (GossipSession  *session,
 
 	priv = GET_PRIV (app);
 
-	gossip_account_param_get (account, "id", &id, NULL);
+	id = gossip_account_get_id (account);
 	gossip_debug (DEBUG_DOMAIN_SESSION, "Error for account:'%s'", id);
 
 	gossip_session_count_accounts (priv->session,
@@ -1682,7 +1632,7 @@ gossip_app_toggle_visibility (void)
 		gtk_widget_hide (priv->window);
 
 		gossip_conf_set_bool (gossip_conf_get (),
-				       GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN, TRUE);
+				      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN, TRUE);
 	} else {
 		gint x, y, w, h;
 
@@ -1705,7 +1655,7 @@ gossip_app_toggle_visibility (void)
 		gossip_window_present (GTK_WINDOW (priv->window), TRUE);
 
 		gossip_conf_set_bool (gossip_conf_get (),
-				       GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN, FALSE);
+				      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN, FALSE);
 	}
 }
 
@@ -1812,7 +1762,7 @@ app_status_icon_create_menu (void)
 	gossip_glade_connect (glade,
 			      app,
 			      "tray_new_message", "activate", app_popup_new_message_cb,
-			      "tray_quit", "activate", app_actions_quit_cb,
+			      "tray_quit", "activate", app_chat_quit_cb,
 			      NULL);
 
 	g_signal_connect (priv->popup_menu_show_list_item, "toggled",
@@ -1849,6 +1799,8 @@ app_status_icon_create (void)
 			  G_CALLBACK (app_status_icon_popup_menu_cb),
 			  app);
 
+	gtk_status_icon_set_visible (priv->status_icon, TRUE);
+
 #ifdef HAVE_LIBNOTIFY
 	gossip_notify_set_attach_status_icon (priv->status_icon);
 #endif
@@ -1856,6 +1808,69 @@ app_status_icon_create (void)
 	app_status_icon_update_tooltip ();
 }
 
+#define MAX_CHECKS 5
+
+static gboolean
+app_status_icon_check_embedded_cb (gpointer user_data)
+{
+	static guint    checks = 1;
+	GossipAppPriv  *priv;
+	GtkRequisition  req;
+	gboolean        hidden;
+	gboolean        embedded;
+
+	priv = GET_PRIV (app);
+
+	embedded = gtk_status_icon_is_embedded (priv->status_icon);
+
+	if (!embedded && checks < MAX_CHECKS) {
+		gossip_debug (DEBUG_DOMAIN_SETUP, 
+			      "Configuring window (status icon not embedded, check %d/%d)", 
+			      checks, MAX_CHECKS);
+		checks++;
+		return TRUE;
+	}
+	
+	if (!embedded) {
+		gossip_debug (DEBUG_DOMAIN_SETUP, 
+			      "Configuring window (status icon not embedded, showing)");
+		hidden = FALSE;
+	} else {
+		GossipConf *conf;
+
+		conf = gossip_conf_get ();
+		gossip_conf_get_bool (conf,
+				      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN,
+				      &hidden);
+		gossip_debug (DEBUG_DOMAIN_SETUP, 
+			      "Configuring window visibility (config says %s)", 
+			      hidden ? "hide it" : "show it");
+	}
+
+	if (!hidden) {
+		gtk_widget_show (priv->window);
+	} else {
+		gdk_notify_startup_complete ();
+	}
+
+	/* This is a hack that sets the minimal size of the window so it doesn't
+	 * allow resizing to a smaller width than the menubar takes up. We must
+	 * set a minimal size, otherwise the window won't shrink beyond the
+	 * longest string in the roster, which will cause the ellipsizing cell
+	 * renderer not to work. FIXME: needs to update this on theme/font
+	 * change.
+	 */
+	gtk_widget_size_request (priv->window, &req);
+	req.width = MAX (req.width, MIN_WIDTH);
+	gtk_widget_set_size_request (priv->window, req.width, -1);
+
+	app_connection_items_update ();
+
+	gossip_debug (DEBUG_DOMAIN_SETUP, "Complete!");
+
+	return FALSE;
+}
+
 static void
 app_status_icon_update_tooltip (void)
 {
@@ -2178,15 +2193,15 @@ app_connection_items_setup (GladeXML *glade)
 	GossipAppPriv *priv;
 
 	const gchar   *widgets_connected[] = {
-		"actions_disconnect",
-		"actions_group_chat",
-		"actions_new_message",
-		"actions_add_contact",
+		"chat_disconnect",
+		"room",
+		"chat_new_message",
+		"chat_add_contact",
 		"edit_personal_information"
 	};
 
 	const gchar   *widgets_disconnected[] = {
-		"actions_connect"
+		"chat_connect"
 	};
 
 	GList         *list;
@@ -2758,36 +2773,19 @@ app_status_icon_flash_maybe_stop (void)
 }
 
 static void
-app_chatroom_auto_connect_update_cb (GossipChatroomManager    *manager,
-				     GossipChatroomProvider   *provider,
-				     GossipChatroom           *chatroom,
-				     GossipChatroomJoinResult  result,
-				     gpointer                  user_data)
+app_chatroom_auto_connect_cb (GossipChatroomManager    *manager,
+			      GossipChatroomProvider   *provider,
+			      GossipChatroom           *chatroom,
+			      gpointer                  user_data)
 {
-	const gchar *name;
+	GossipGroupChat *chat;
 
-	name = gossip_chatroom_get_name (chatroom);
+	gossip_debug (DEBUG_DOMAIN_CHATROOMS, 
+		      "Auto connect for room:'%s'", 
+		      gossip_chatroom_get_name (chatroom));
 
-	switch (result) {
-	case GOSSIP_CHATROOM_JOIN_NICK_IN_USE:
-	case GOSSIP_CHATROOM_JOIN_NEED_PASSWORD:
-	case GOSSIP_CHATROOM_JOIN_TIMED_OUT:
-	case GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST:
-	case GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR:
-		gossip_debug (DEBUG_DOMAIN_CHATROOMS, "Auto connect update: failed for room:'%s'", name);
-		gossip_chatrooms_window_show (NULL, TRUE);
-		break;
-
-	case GOSSIP_CHATROOM_JOIN_OK:
-	case GOSSIP_CHATROOM_JOIN_ALREADY_OPEN:
-		gossip_debug (DEBUG_DOMAIN_CHATROOMS, "Auto connect update: success for room:'%s'", name);
-		gossip_group_chat_new (provider,
-				       gossip_chatroom_get_id (chatroom));
-		break;
-
-	case GOSSIP_CHATROOM_JOIN_CANCELED:
-		break;
-	}
+	chat = gossip_group_chat_new (provider, chatroom);
+/* 	g_object_unref (chat); */
 }
 
 static void
